import { filter, first, map, takeLast } from 'rxjs/operators';
import { isObservable, Observable } from 'rxjs';
import { isPromise } from 'rxjs/internal-compatibility';

/**
 * Returns an Observable that emits the first truthy value from a source.
 *
 * @category Boolean Filters
 *
 * @typeParam T The value contained in the source Observable
 *
 * @param predicate Optional [[PredicateFn]] function to compared the values against
 *
 * @example
 * Return the first truthy string
 * ```ts
 * const input = ['', '', 'Hello', 'RxJS', 'Ninja'];
 * from(input).pipe(firstTruthy()).subscribe();
 * ```
 * Output: `'Hello'`
 *
 * @example
 * Return the first truthy number
 * ```ts
 * const input = [0, 1, 2, 3, 4, 5, 6];
 * from(input).pipe(firstTruthy()).subscribe();
 * ```
 * Output: `1`
 *
 * @example
 * Return the first truthy number with predicate
 * ```ts
 * const input = [0, 1, 2, 3, 4, 5, 6];
 * from(input).pipe(firstTruthy((value) => value % 2 === 0)).subscribe();
 * ```
 * Output: `2`
 *
 * @returns Observable that emits the first truthy value
 */

function firstTruthy(predicate) {
  return function (source) {
    return source.pipe(filter(function (value) {
      return predicate ? Boolean(value) && predicate(value) : Boolean(value);
    }), first());
  };
}

/**
 * Returns an Observable that emits only truthy values from a source
 *
 * @category Boolean Filters
 *
 * @typeParam T The value contained in the source Observable
 *
 * @param predicate Optional [[PredicateFn]] function to compared the values against
 *
 * @example
 * Returns a string value from a source where the string is truthy
 * ```ts
 * const input = ['', 'RxJS', '', 'Ninja', ''];
 * from(input).pipe(filterTruthy()).subscribe();
 * ```
 * Output: `'RxJS', 'Ninja'`
 *
 * @example
 * Returns a string value from a source where the string passes the predicate
 * ```ts
 * const input = ['', 'RxJS', '', 'Ninja', ''];
 * from(input).pipe(filterTruthy(v => v.length < 5 )).subscribe();
 * ```
 * Output: `'RxJS'`
 *
 * @example
 * Returns a number value from a source where the number is truthy
 * ```ts
 * const input = [0, 1, 2, 3, 4, 5, 6];
 * from(input).pipe(filterTruthy()).subscribe();
 * ```
 * Output: `2, 4`
 *
 * @example
 * Returns a number value from a source where the number passes the predicate
 * ```ts
 * const mod3 = (num: number) => num % 3 === 0
 * const input = [0, 1, 2, 3, 4, 5, 6];
 * from(input).pipe(filterTruthy(mod3)).subscribe();
 * ```
 * Output: `3, 6`
 *
 * @returns Observable that emits only truthy values or values that pass the optional [[PredicateFn]]
 */

function filterTruthy(predicate) {
  return function (source) {
    return source.pipe(filter(function (value) {
      return predicate ? Boolean(value) && predicate(value) : Boolean(value);
    }));
  };
}

/**
 * Returns an Observable that emits a flipped boolean value from the source value
 *
 * @category Boolean Modify
 *
 * @example
 * Returns flipped boolean values
 * ```ts
 * const input = [false, true, false];
 * from(input).pipe(flip()).subscribe()
 * ```
 * Output: `true, false, true`
 *
 * @returns Observable that emits a boolean where the source Observable value have been flipped
 */

function flip() {
  return function (source) {
    return source.pipe(map(function (value) {
      return !value;
    }));
  };
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __read(o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
}

function __spread() {
    for (var ar = [], i = 0; i < arguments.length; i++)
        ar = ar.concat(__read(arguments[i]));
    return ar;
}

/**
 * Returns an Observable that emits boolean values from passed source of values.
 *
 * @category Boolean Observables
 *
 * @typeParam T The type or types to be used to create boolean values from
 *
 * @param args Input values to create the Observable source from
 *
 * @example
 * Create an Observable that emits a single boolean value from a string, flip the value
 * ```ts
 * fromBoolean('Hello RxJS Ninja').pipe(flip()).subscribe();
 * ```
 * Output: `false`
 *
 * @example
 * Create an Observable that emits a single boolean from an array of values
 * ```ts
 * fromBoolean(['', 'Hello', 'RxJS', '']).subscribe();
 * ```
 * Output: `false, true, true, false`;
 *
 * @returns Observable that emits boolean values of the source value

 */

function fromBoolean() {
  var args = [];

  for (var _i = 0; _i < arguments.length; _i++) {
    args[_i] = arguments[_i];
  }

  if (isObservable(args[0])) {
    return args[0].pipe(map(Boolean));
  } else if (isPromise(args[0])) {
    return new Observable(function (subscriber) {
      args[0].then(function (value) {
        if (!subscriber.closed) {
          subscriber.next(value);
          subscriber.complete();
        }
      }, function (err) {
        return subscriber.error(err);
      });
    });
  } else {
    var value_1 = Array.isArray(args[0]) ? args[0] : __spread(args);
    value_1 = value_1.map(Boolean);
    return new Observable(function (subscriber) {
      for (var i = 0; i < value_1.length; i++) {
        subscriber.next(value_1[i]);
      }

      subscriber.complete();
    });
  }
}

/**
 * Returns an Observable that emits the last truthy value from a source.
 *
 * @category Boolean Filters
 *
 * @typeParam T The value contained in the source Observable
 *
 * @param predicate Optional [[PredicateFn]] function to compared the values against
 *
 * @example
 * Return the last truthy string
 * ```ts
 * const input = ['', '', 'Hello', 'RxJS', 'Ninja'];
 * from(input).pipe(lastTruthy()).subscribe();
 * ```
 * Output: `'Ninja'`
 *
 * @example
 * Return the last truthy number
 * ```ts
 * const input = [0, 1, 2, 3, 4, 5, 6];
 * from(input).pipe(lastTruthy()).subscribe();
 * ```
 * Output: `6`
 *
 * @example
 * Return the last truthy number with predicate
 * ```ts
 * const input = [0, 1, 2, 3, 4, 5, 6];
 * from(input).pipe(lastTruthy((value) => value % 2 === 0 && value % 3 !== 0)).subscribe();
 * ```
 * Output: `4`
 *
 * @returns Observable that emits the last truthy value
 */

function lastTruthy(predicate) {
  return function (source) {
    return source.pipe(filter(function (value) {
      return predicate ? Boolean(value) && predicate(value) : Boolean(value);
    }), takeLast(1));
  };
}

/**
 * @packageDocumentation
 * @module Boolean
 */

/**
 * Takes a string or number and converts to an array of numbers in reverse order
 * @param value
 * @private
 * @internal
 */
function reverseLuhnNumbers(value) {
  return (typeof value === 'number' ? value.toString(10) : value).split('').reverse().map(function (val) {
    return parseInt(val, 10);
  });
}
/**
 * Generate a moulus check number from input
 * @param valueArray
 * @private
 * @internal
 */

function createLuhnModulus(valueArray) {
  var lastDigit = valueArray.splice(0, 1)[0];
  /* istanbul ignore next */

  var sum = valueArray.reduce(function (acc, val, i) {
    return i % 2 !== 0 ? acc + val : acc + val * 2 % 9 || 9;
  }, 0);
  return sum + lastDigit;
}

/**
 * Returns an Observable that emits a boolean value.  The source should emit a number or string that can be checked
 * with the {@link https://en.wikipedia.org/wiki/Luhn_algorithm|Luhn Algorithm} used to validate identification numbers
 * such as bank cards and IMEI numbers.
 *
 * @category Boolean Validation
 *
 * @example
 * Returns if the value passes the Luhn check.
 * ```ts
 * const input = ['4485275742308327', '1111222233334444', '111133332224444'];
 * from(input).pipe(luhnCheck()).subscribe();
 *```
 * Output: `[true, true, false]`
 *
 * @returns Observable that emits an boolean if the source value passes the Luhn check
 */

function luhnCheck() {
  return function (source) {
    return source.pipe(map(reverseLuhnNumbers), map(createLuhnModulus), map(function (value) {
      return value % 10 === 0;
    }));
  };
}

/**
 * Returns an Observable that emits a boolean value from a source value of any type
 *
 * @category Boolean Observables
 *
 * @typeParam T The value contained in the source Observable
 *
 * @param predicateFn Optional [[PredicateFn]] function to compared the values against
 *
 * @example
 * Returns boolean values for source strings
 * ```ts
 * const input = ['', 'Hello', 'RxJS', ''];
 * from(input).pipe(toBoolean()).subscribe()
 * ```
 * Output: `false, true, true, false`
 *
 * @example
 * Returns boolean values for source strings with predicate
 * ```ts
 * const input = ['', 'Hello', 'RxJS', ''];
 * from(input).pipe(toBoolean(v => v.length < 5)).subscribe()
 * ```
 * Output: `true, false, true, true`
 *
 * @returns Observable that emits a boolean value of a source value
 */

function toBoolean(predicateFn) {
  return function (source) {
    return source.pipe(map(function (value) {
      return predicateFn ? predicateFn(value) : Boolean(value);
    }));
  };
}

export { filterTruthy, firstTruthy, flip, fromBoolean, lastTruthy, luhnCheck, toBoolean };
